@@ -31,7 +31,6 @@ import "C"
 import (
 	"fmt"
 	"io"
-	"io/ioutil"
 	"log"
 	"log/syslog"
 	"os"
@@ -132,7 +131,7 @@ func parseArgs(argc C.int, argv **C.char) map[string]bool {
 // syslog.
 func setupLogging(args map[string]bool) io.Writer {
 	log.SetFlags(0) // Syslog already includes time data itself
-	log.SetOutput(ioutil.Discard)
+	log.SetOutput(io.Discard)
 	if args[debugFlag] {
 		debugWriter, err := syslog.New(syslog.LOG_DEBUG, moduleName)
 		if err == nil {
@@ -142,7 +141,7 @@ func setupLogging(args map[string]bool) io.Writer {
 
 	errorWriter, err := syslog.New(syslog.LOG_ERR, moduleName)
 	if err != nil {
-		return ioutil.Discard
+		return io.Discard
 	}
 	return errorWriter
 }
@@ -179,8 +178,10 @@ func loginProtector(handle *pam.Handle) (*actions.Protector, error) {
 }
 
 // policiesUsingProtector searches all the mountpoints for any policies
-// protected with the specified protector.
-func policiesUsingProtector(protector *actions.Protector) []*actions.Policy {
+// protected with the specified protector.  If provisioned is true, then only
+// policies provisioned by the target user are returned; otherwise only policies
+// *not* provisioned by the target user are returned.
+func policiesUsingProtector(protector *actions.Protector, provisioned bool) []*actions.Policy {
 	mounts, err := filesystem.AllFilesystems()
 	if err != nil {
 		log.Print(err)
@@ -213,9 +214,23 @@ func policiesUsingProtector(protector *actions.Protector) []*actions.Policy {
 				continue
 			}
 
-			if policy.UsesProtector(protector) {
-				policies = append(policies, policy)
+			if !policy.UsesProtector(protector) {
+				continue
 			}
+			if provisioned {
+				if !policy.IsProvisionedByTargetUser() {
+					log.Printf("policy %s not provisioned by %v",
+						policy.Descriptor(), ctx.TargetUser.Username)
+					continue
+				}
+			} else {
+				if policy.IsProvisionedByTargetUser() {
+					log.Printf("policy %s already provisioned by %v",
+						policy.Descriptor(), ctx.TargetUser.Username)
+					continue
+				}
+			}
+			policies = append(policies, policy)
 		}
 	}
 	return policies
